/*
 * 04_Data_Visualizer.cpp
 * Created: 2021
 *
 * Pindescription Arduino Nano
 * No.	Func.	PIN-MC	Used as In/Out Res. Pindefinition
 * D0	RX		PD0		RX			In
 * D1	TX		PD1		TX			In
 * D2	INT0	PD2		US-Echo_L	In
 * D3	INT1	PD3		US-Echo_R	In
 * D4	Digit	PD4		US-Trigger_L Out
 * D5	OC0B	PD5		PWM-B		Out
 * D6	OC0A	PD6		PWM-A		Out
 * D7	Digit	PD7		US-Trigger_R Out	->	DDRD = 0b11110000
 * D8	Digit	PB0		Motor-B		Out
 * D9	Digit	PB1		Motor-A		Out
 *D10	Digit	PB2		LS_LED		Out
 *D11	Unused	PB3					Out
 *D12	Unused	PB4					Out
 *D13	LED		PB5		LED			Out	->	DDRB = 0b11111111
 *A0	Ana		PC0		Line-L3		In
 *A1	Ana		PC1		Line-L2		In
 *A2	Ana		PC2		Line-L1		In
 *A3	Ana		PC3		Line-L0		In
 *A4	Ana		PC4		Line-R0		In
 *A5	Ana		PC5		Line-R1		In	->	DDRC = 0b00000000
 *A6	Ana		ADC6	Line-R2		In
 *A7	Ana		ADC7	Line-R3		In
 */
#define F_CPU 16000000UL //16MHz required for delay
#include <avr/io.h> //Input/Output library
#include <util/delay.h> //Needed for using _delay_...
#include <avr/interrupt.h> //External and internal Interrupts
#include "Init.h"
#include "USART_Functions.h"

//Definitions
#define FOSC				16000000 // Clock Speed
#define BAUD				76800//Baud rate, normally used by Arduino 9600, possible rates: n * 9600
#define MYUBRR				FOSC/16/BAUD-1 //Calculate my baud rate

//Macros
#define	LED_ON				PORTB |= (1 << PORTB5) //Push PORTB pin 5 high (LED, pin D13 "Nano")
#define	LED_OFF				PORTB &= ~(1 << PORTB5) //~ Not Operator
#define	LED_TOGGLE			PINB |= (1<<PINB5) //Toggle Arduino Nano pin D13

//Variables
unsigned char z; //For tests

unsigned int T2ck; //Counter TMR2 overflow
/************************************************************************/
/* Hier eure Variable (unsigned char) deklarieren                       */
/************************************************************************/

unsigned char Data[10]; //Number of datas displayed on Data Visualizer

//Funktions, declaration
void Init (void); //Initialize Microcontroller
//void USART_Transmit(unsigned char); //Transmit ASCII to Data Visualizer
void USART_Init(unsigned int); //Initialize USART Transmission/Reception
//void Transmit_literal (char); //Transmit literal to Data Visualizer Monitor
//void Transmit_String (const char); //Transmit letters to Data Visualizer Monitor
void Data_Visualizer (void); //Define datas to be vizualized
/************************************************************************/
/* Hier eure Funktion deklarieren                                       */
/************************************************************************/

//Main routine
int main(void) {
	Init(); //Initalize the MC
	USART_Init(MYUBRR); //Transmision/reception initialize
	sei(); //Enable all interrupts
	//cli(); //Clear all interrupts
	while (1) {
		//Transmit_String("Text: May be as long as you want, but lasts and lasts and...! \n");
		Data_Visualizer(); //Define the values to be displayed and transmit them
		//_delay_ms(10);
		/************************************************************************/
		/* Hier eure Funktion aufrufen                                          */
		/************************************************************************/
	}
}
/***********************************************************************************************/
//Other functions
/************************************************************************/
/* Hier eure Funktion definieren                                        */
/************************************************************************/
//Datas transmitted to the Data Visualizer
void Data_Visualizer (void) {
	Data[0] = z++; //First value at the console. Same as z = z + 1. Just to see how "fast" the transmission is
	Data[1] = (unsigned char) (T2ck >> 8); //Casting integer to character, most significant 8 bits
	Data[2] = (unsigned char) T2ck; //Casting integer to character, least significant 8 bits
	/************************************************************************/
	/* Hier euren Wert anzeigen lassen                                      */
	/* Und, ganz wichtig die 3 in der for-Schleife durch 4 ersetzen!!!      */                                     
	/************************************************************************/
	for (unsigned char i = 0; i < 3; i++) {
		Transmit_literal(Data[i]); //Three digit literal unsigned, three columns
		//_delay_ms(10);
	}
	USART_Transmit('\n'); //New line
	//USART_Transmit(10); //ASCII for "line feed", LF
	//USART_Transmit('\r'); //Cariage return (CR) or 13
}

/***********************************************************************************************/

/***********************************************************************************************/
//Interrupt service routines (ISR)

//TMR2 overflow Interrupt
//To time several works
ISR(TIMER2_OVF_vect) { //Prescaler TMR2 1:8 = Interrupt every 128 us
	//Toggeling the LED, just to see, if the MC is working
	T2ck++; //Counter TMR2 OVF
	if(T2ck == 2000) { //Every 250ms
		T2ck = 0; //Dont forget!!
		LED_TOGGLE; //See above
	}
}





